#!/usr/bin/perl
#
# need to install support for ldap-perl first:
#
# perl -MCPAN -e shell
# install Bundle::Net::LDAP
# 
# (c) 2002 jan-ake.ronnblom@skeria.skelleftea.se
#
# makeaccess.pl version: TEST.1
#
# Remember there is a sizelimit on the MS LDAP server which only gives you 1000 objects/attributes.
#
# In case of more than 1000 objects you could use Paged access or VLV (better).
# In case of more than 1000 attributes on a object (like a group with 1000 members) you need to use
# the range qualifier in your LDAP search.
#
# example:
#
# $mesg = $ldap->search(base => $base,
#                    scope => $scope,
#                    filter => $filter,
#                    attrs => ['member;range=0-999']
#                    );
#
# $entr = $mesg->entry(0);
#
# attrs can be ['member;range=0-999'] even if the group/object contains fewer than
# 1000 users/attributes.
#
# if the answer ($entr->attributes) contains 'member;range=0-*' (notice the *) then there are no
# more attributes. Otherwise I have to modify attrs ('member;range=1000-1999']) and search again
# until no more attributes are found (range=X-*).
#
# configuration for squid.conf:
# 
# auth_param ntlm program /usr/lib/squid/fakeauth_auth
# auth_param ntlm children 5
# auth_param ntlm max_challenge_reuses 0
# auth_param ntlm max_challenge_lifetime 2 minutes
#
# # allow access to *.skelleftea.se and .*skelleftea.org
# acl skelleftea dstdomain .skelleftea.se .skelleftea.org
#
# hmm fungerar detta?
# acl internetaccess proxy_auth "/etc/squid/iagrupp.txt"
#
# http_access allow skelleftea
# http_access allow internetaccess
#

use Net::LDAP qw(:all);
use Net::LDAP::Control::Paged;
use Net::LDAP::Constant qw( LDAP_CONTROL_PAGED );

$NETBIOSDOMAIN="LABB";
$ldapuser = "myuser\@labb.skelleftea.se";
$ldappasswd = "mypassword";
$host = "192.168.1.1";
#$port = "389";
$port = "3268"; # global catalog

# dont ask the LDAP database, ask the global catalog instead (should be faster since GC is indexed)

$base = "DC=labb,DC=skelleftea,DC=se";
$basescope = "sub";
$basefilter = "objectCategory=CN=Group,CN=Schema,CN=Configuration,DC=labb,DC=skelleftea,DC=se";
my @baseattrs = ['distinguishedName'];

$ldap = Net::LDAP->new($host, port => $port) or die "$@";

$mesg = $ldap->bind(dn => $ldapuser, password => $ldappasswd, version => 3);
$page = Net::LDAP::Control::Paged->new( size => 100 );

# search args
@args = ( base    => $base,
          filter  => $basefilter,
          attrs   => @baseattrs,
          control => [ $page ]);

while(1) {
	$mesg = $ldap->search( @args );
	$entries = $mesg->count;
	for($i = 0 ; $i < $entries ; $i++) {
		$entry = $mesg->entry($i);
		$dn = $entry->get_value('distinguishedName');

		# remember: ignore case!
		if ($dn=~ /internetaccess/i)
		{
			getmembers($ldap, $dn);
		}
	}
	$mesg->code and last;
	my($resp) = $mesg->control( LDAP_CONTROL_PAGED ) or last;
	$cookie = $resp->cookie or last;
	$page->cookie($cookie);
}


$ldap->unbind;

sub getmembers
{
	my ($ldap, $dn) = @_;

	local($base, $scope, $filter, @attrs, $mesg);
	local($entr, $entries);

	$base = $dn;
	$scope = "base";
	$filter = "objectClass=*";
	@attrs = ['member;range=0-999'];
	my $found_all = 0;
	my $counter = 1;
	
	while($found_all == 0)
	{
		$mesg = $ldap->search(base => $base,
	        	scope => $scope,
	                filter => $filter,
	               attrs => @attrs # ['member;range=0-999']
	                );

		$entries = $mesg->count;

		$entr = $mesg->entry(0);

		my @attrib = $entr->attributes;
		$attrib = $attrib[0];

		if($attrib)
		{

			@val = $entr->get_value($entr->attributes);
			foreach $val ( @val )
			{

#				$val = 'CN=jar0101,OU=Students,OU=FR,DC=labb,DC=skelleftea,DC=se';
#				$val =~ /^CN=(\w\w[\w|\-|\s][\w|\-]\w*),.*/;

# YOU NEED TO CHANGE THIS LINE BELOW TO MAKE IT WORK AGAINST YOUR MS-AD
# OR USE THE CALL TO getSAMaccount EVERY TIME (MUCH SLOWER)
# FOR NOW IT USES ONLY THE getSAMaccount CALL - YOU NEED TO CHANGE THE
# TRIPLE # (###) LINES TO MATCH YOUR AD.

# UNCOMMENT THIS:

###				$val =~ /^CN=([\w\-\s]*),.*/;
###				$match = $1;

# All my objects(users) in the Active Directory has the DN = SAMAccountname which makes it
# possible for me to not check the value of the SAMAccountName. They are created by a script
# which makes sure that the DN is SAMAccountName.
#
# On 6000+ objects this script now takes 1.6s and if I need to check the SAMAccountNAme on
# every object it would take more than 25s.
#

# Note: I do have to check some objects which is manually created. They have a space in their
# name and those I pass to the getSAMaccount routing.
#				print "=== above is wrong (dN != SAM)\n" if($match =~ / /);

# UNCOMMENT THIS:

###
###				if($match =~ / /)
###				{
###					getSAMaccount($ldap, $val);
###				}
###				else
###				{
###					print "$NETBIOSDOMAIN\\$match\n"
###				}

# COMMENT THIS:

				getSAMaccount($ldap, $val);

			}

			if($attrib =~ /\*/)
			{
				$found_all = 1;
			}
			else
			{
				$newattr = "member;range=${counter}000-${counter}999";
				$counter++;
				@attrs = [ $newattr ];
			}
		}
		else
		{
			$found_all = 1;
		}

	}


}

sub getSAMaccount
{
	my ($ldap, $dn) = @_;

	local($base, $scope, $filter, @attrs, $mesg);
	local($entr, $entries, @attrib, $attrib, @val);

	$base = $dn;
	$scope = "base";
	$filter = "(objectCategory=CN=Person,CN=Schema,CN=Configuration,dc=labb,dc=skelleftea,dc=se)";
	@attrs = ['sAMAccountName'];

	$mesg = $ldap->search(base => $base,
        	scope => $scope,
                filter => $filter,
                attrs => @attrs
                );

	$entries = $mesg->count;

	$entr = $mesg->entry(0);

	$val = $entr->get_value($entr->attributes);
	print "$NETBIOSDOMAIN\\$val\n";

}

### END


